@@ -40,6 +40,9 @@ sub process_file
     my $soi = my_read($fh, 2);
     die "SOI missing" unless $soi eq "\xFF\xD8";
 
+    $info->push_info(0, "FileMediaType" => "image/jpeg");
+    $info->push_info(0, "FileExt" => "jpg");
+
     while (1) {
         my($ff, $mark, $len) = unpack("CCn", my_read($fh, 4));
         last if $ff != 0xFF;
@@ -57,29 +60,34 @@ sub process_chunk
     if ($mark == 0xFE) {
         $info->push_info(0, Comment => $data);
     }
+    elsif ($mark >= 0xE0 && $mark <= 0xEF) {
+        process_app($info, $mark, $data);
+    }
     elsif ($sof{$mark}) {
         my($precision, $height, $width, $num_comp) =
             unpack("CnnC", substr($data, 0, 6, ""));
-	$info->push_info(0, "ImageType", "JPEG ". $sof{$mark});
+	$info->push_info(0, "JPEG_Type", $sof{$mark});
 	$info->push_info(0, "ImageWidth", $width);
-	$info->push_info(0, "ImageLength", $height);
-        $info->push_info(0, "Precision", "$num_comp x $precision");
-=for comment
-        my $i = 1;
-        while (length($data)) {
-            my($comp_id, $hv, $qtable) =
-                unpack("CCC", substr($data, 0, 3, ""));
-            printf "  Color component %d: id=%d, hv=%d, qtable=%d\n",
-                $i, $comp_id, $hv, $qtable;
-        }
-        continue {
-            $i++;
-        }
-=cut
+	$info->push_info(0, "ImageHeight", $height);
 
-    }
-    elsif ($mark >= 0xE0 && $mark <= 0xEF) {
-        process_app($info, $mark, $data);
+	for (1..$num_comp) {
+	    $info->push_info(0, "BitsPerSample", $precision);
+	}
+	$info->push_info(0, "SamplesPerPixel" => $num_comp);
+	if ($num_comp == 1) {
+	    $info->push_info(0, "ColorType" => "Gray");
+	}
+	elsif ($num_comp == 3) {
+	    $info->push_info(0, "ColorType" => "RGB");  # or YCC??
+	}
+
+	if (0) {
+	    while (length($data)) {
+		my($comp_id, $hv, $qtable) =
+		    unpack("CCC", substr($data, 0, 3, ""));
+		$info->push_info(0, "ColorComponents", [$comp_id, $hv, $qtable]);
+	    }
+	}
     }
 }
 
@@ -115,13 +123,18 @@ sub process_app0_jfif
     my($ver_hi, $ver_lo, $units, $x_density, $y_density, $x_thumb, $y_thumb) =
 	unpack("CC C nn CC", substr($data, 0, 9, ""));
     $info->push_info(0, "JFIF_Version", sprintf("%d.%02d", $ver_hi, $ver_lo));
-    $info->push_info(0, "JFIF_Density",
-		     sprintf("%dx%d %s", $x_density, $y_density,
-			     { 0 => "pixels",
-			       1 => "dpi",
-			       2 => "dots per cm"
-			     }->{$units} || "(unit $units)"
-			    ));
+    if ($x_density == $y_density) {
+	$info->push_info(0, "Resolution" => $x_density);
+    }
+    else {
+	$info->push_info(0, "XResolution" => $x_density);
+	$info->push_info(0, "YResolution" => $y_density);
+    }
+    $info->push_info(0, "ResolutionUnit" => { 0 => "pixels",
+					      1 => "dpi",
+					      2 => "dpcm"
+					    }->{$units} || $units);
+
     if ($x_thumb || $y_thumb) {
 	$info->push_info(1, "ImageWidth", $x_thumb);
 	$info->push_info(1, "ImageLength", $y_thumb);
@@ -149,8 +162,8 @@ sub process_app1_exif
 	return;
     }
 
-    require Image::TIFF::Exif;
-    my $t = Image::TIFF::Exif->new(\$data);
+    require Image::TIFF;
+    my $t = Image::TIFF->new(\$data);
 
     for my $i (0 .. $t->num_ifds - 1) {
 	my $ifd = $t->ifd($i);
@@ -158,7 +171,6 @@ sub process_app1_exif
 	    $info->push_info($i, $_->[0], $_->[3]);
 	}
     }
-
 }
 
 1;
