@@ -26,9 +26,30 @@ sub process_file
     die "Bad PNG signature"
 	unless $signature eq "\x89PNG\x0d\x0a\x1a\x0a";
 
+    $info->push_info(0, "FileMediaType" => "image/png");
+    $info->push_info(0, "FileExt" => "png");
+
+    my @chunks;
+
     while (1) {
         my($len, $type) = unpack("Na4", my_read($fh, 8));
-        $info->push_info(0, "PNG_Chunks", $type) unless $type eq "IDAT";
+
+	if (@chunks) {
+	    my $last = $chunks[-1];
+	    $last =~ s/\s(\d+)$//;
+	    my $count = $1 || 1;
+	    if ($last eq $type) {
+		$count++;
+		$chunks[-1] = "$type $count";
+	    }
+	    else {
+		push(@chunks, $type);
+	    }
+	}
+	else {
+	    push(@chunks, $type);
+	}
+
         last if $type eq "IEND";
         my $data = my_read($fh, $len + 4);
 	my $crc = unpack("N", substr($data, -4, 4, ""));
@@ -36,11 +57,11 @@ sub process_file
 	    my($w, $h, $depth, $ctype, $compression, $filter, $interlace) =
 		unpack("NNCCCCC", $data);
 	    $ctype = {
-		      0 => "GrayScale",
-		      2 => "TrueColor",
-		      3 => "IndexedColor",
-		      4 => "GrayScale/Alpha",
-		      6 => "TrueColor/Alph",
+		      0 => "Gray",
+		      2 => "RGB",
+		      3 => "Indexed-RGB",
+		      4 => "GrayA",
+		      6 => "RGBA",
 		     }->{$ctype} || $ctype;
 
 	    $compression = "Deflate" if $compression == 0;
@@ -48,16 +69,46 @@ sub process_file
 	    $interlace = "Adam7" if $interlace == 1;
 
 	    $info->push_info(0, "ImageWidth", $w);
-	    $info->push_info(0, "ImageLength", $h);
-	    $info->push_info(0, "BitDepth", $depth);
+	    $info->push_info(0, "ImageHeight", $h);
+	    $info->push_info(0, "BitsPerSample", $depth);
 	    $info->push_info(0, "ColorType", $ctype);
 	    $info->push_info(0, "Compression", $compression);
-	    $info->push_info(0, "Filter", $filter);
-	    $info->push_info(0, "Interlace", $interlace);
+	    $info->push_info(0, "PNG_Filter", $filter);
+	    $info->push_info(0, "Interlace", $interlace)
+		if $interlace;
+	}
+	elsif ($type eq "PLTE") {
+	    my @table;
+	    while (length $data) {
+		push(@table, sprintf("#%02x%02x%02x",
+				     unpack("C3", substr($data, 0, 3, ""))));
+	    }
+	    $info->push_info(0, "RGB_Palette" => \@table);
 	}
 	elsif ($type eq "gAMA" && $len == 4) {
 	    $info->push_info(0, "Gamma", unpack("N", $data)/100_000);
 	}
+	elsif ($type eq "pHYs" && $len == 9) {
+	    my($res_x, $res_y, $unit) = unpack("NNC", $data);
+	    if (0 && $unit == 1) {
+		# convert to dpi
+		$unit = "dpi";
+		for ($res_x, $res_y) {
+		    $_ *= 0.0254;
+		}
+	    }
+	    if ($res_x == $res_y) {
+		$info->push_info(0, "Resolution" => $res_x);
+	    }
+	    else {
+		$info->push_info(0, "XResolution" => $res_x);
+		$info->push_info(0, "YResolution" => $res_y);
+	    }
+	    if ($unit) {
+		$unit = "dpm" if $unit == 1;
+		$info->push_info(0, "ResolutionUnit" => $unit);
+	    }
+	}
 	elsif ($type eq "tEXt") {
 	    my($key, $val) = split(/\0/, $data, 2);
 	    # XXX should make sure $key is not in conflict with any
@@ -65,11 +116,13 @@ sub process_file
 	    $info->push_info(0, $key, $val);
 	}
 	elsif ($type eq "tIME" && $len == 7) {
-	    $info->push_info(0, "DateTime",
+	    $info->push_info(0, "LastModificationTime",
 			     sprintf("%04d-%02d-%02d %02d:%02d:%02d",
 				     unpack("nC5", $data)));
 	}
     }
+
+    $info->push_info(0, "PNG_Chunks", @chunks);
 }
 
 1;
